import os, time, subprocess, signal, re, sys

ON_WINDOWS = (sys.platform == 'win32')
SIGNALS = {
    signal.SIGINT: 'SIGINT',
    signal.SIGTERM: 'SIGTERM',
}
if ON_WINDOWS:
    SIGNALS[signal.SIGBREAK] = 'SIGBREAK'

child = None

def term(proc):
    """Send a SIGTERM/SIGBREAK to *proc* and wait for it to terminate."""
    if ON_WINDOWS:
        proc.send_signal(signal.CTRL_BREAK_EVENT)
    else:
        proc.terminate()
    proc.wait()
    
def sigHandler(signum, frame):
    """Stop the sub-process *child* if *signum* is SIGTERM. Then terminate."""
    try:
        print('received signal=%s' % (SIGNALS[signum]))
        if child and (ON_WINDOWS or signum != signal.SIGINT):
            # Forward SIGTERM on Linux or any signal on Windows
            print('forwarding received signal=%s to child' % (SIGNALS[signum]))
            term(child)
    except:
        traceback.print_exc()
    finally:
        sys.exit()
  
def subproc(args, env=None):
    """Create and return a new subprocess named *name*."""
    kwargs = {}
    if ON_WINDOWS:
        kwargs['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
    else:
        kwargs['preexec_fn'] = os.setsid
    proc = subprocess.Popen(args, env=env, **kwargs)
    return proc


def runCriblCommand(args):
    splunk_home = os.getenv("SPLUNK_HOME", "")
    splunk_cmd = os.path.join(splunk_home, "bin","splunk")

    #1. see if we can find a better node version than the one shipped with Splunk (from which we need --harmony)
    node_cmd = [splunk_cmd, "cmd", "node", "--harmony"]
    cmd_env = os.environ.copy()
    splunk_node='v0.0.0'

    try:
        splunk_node=subprocess.check_output(node_cmd + ["--version"])
    except:
        pass

    try:
        env = os.environ.copy()
        # remove SPLUNK_HOME/bin from path, otherwise we'll compare splunk node vs itself
        env["PATH"] = os.pathsep.join(env["PATH"].split(os.pathsep)[1:])
        
        system_node=subprocess.check_output(["node", "--version"], env=env)
        versRegex = re.compile("^v([0-9]+)\.([0-9]+)\.([0-9]+)$")
        sysV = versRegex.match(system_node.strip())
        splV = versRegex.match(splunk_node.strip())
        if sysV is not None and splV is not None:
            if int(sysV.group(1)) > int(splV.group(1)) or int(sysV.group(2)) > int(splV.group(2)) or int(sysV.group(3)) > int(splV.group(3)):
                print "using system node, version=%s, splunk_node=%s" % (system_node.strip(), splunk_node.strip())
                node_cmd = ['node']
                cmd_env = env
    except: 
        pass

    # if we're using Splunk's node and it's version is 1.0.0 (sentinel val) then barf!
    if len(node_cmd) > 1 and splunk_node == 'v0.0.0':
        print "No suitable NodeJs version found. Please ensure NodeJs is installed and node is in the PATH"
        sys.exit(42)

    #2. finally, start Cribl ... start it as a separate process, poll its status periodically, that way 
    # this process is able to receive and propagate signals to it. It sucks that os.exec doesn't do the right thing in windows, ohh well!
    signal.signal(signal.SIGINT, sigHandler)
    if ON_WINDOWS:
        signal.signal(signal.SIGBREAK, sigHandler)
    else:
        signal.signal(signal.SIGTERM, sigHandler)

    cribl_cmd = node_cmd + [os.path.join(splunk_home, "etc", "apps", "cribl", "bin", "cribl.bundle.js")] + args
    cmd_env["CRIBL_PY"] = "1"
    global child
    child = subproc(cribl_cmd, cmd_env)
    while True:
        time.sleep(0.1)
        retcode = child.poll()
        if retcode is not None:
            sys.exit(retcode)

if __name__ == "__main__":
    runCriblCommand(sys.argv[1:])